﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Text;
using System.Threading;

namespace MyRoboter
{
    public class CRoboter
    {
        private System.ComponentModel.IContainer components = null;
        public System.IO.Ports.SerialPort serialPort1;
        public enum mot : byte { stopp, vor, rueck, links, rechts, linksdreh, rechtsdreh, linksrueck, rechtsrueck };
        [FlagsAttribute]
        public enum Sens : byte { IRR = 1, IRM = 2, IRL = 4, EndR = 8, EndL = 16, RflxL = 32, RflxM = 64, RflxR = 128 };

        int Warte = 10;                                         //Zeit in ms (Bei schnelleren PC`s setzen,
        //damit die Programme korrekt funktionieren)

        private void Init()
        {
            this.components = new System.ComponentModel.Container();
            this.serialPort1 = new System.IO.Ports.SerialPort(this.components);
            this.serialPort1.BaudRate = 9600;
            this.serialPort1.WriteTimeout = 500;
            this.serialPort1.ReadTimeout = 500;
            this.serialPort1.PortName = "COM1";                   //Hier wird der COM-Port-Name definiert.
        }                                //COM1 - COMxx

        public CRoboter()       // Konstruktor
        {
            Init();             //COM-Port wird initialisiert
        }

        //Es wird die Richtung und Geschwindigkeit (speed) für die Motoren übergeben
        //0 = aus, 10 = max. Geschwindigkeit
        public void motor(mot richtung, Int32 speed)
        {
            byte[] daten = new byte[3];
            int temp;

            daten[0] = 1;       //1 spricht im Roboter die Bewegungsroutinen an

            switch (richtung)       // Setzt die Bitkombinationen für die GS-Motoren
            {                       // des Roboters
                case mot.stopp: daten[1] = 0x00; break;
                case mot.vor: daten[1] = 0x0A; break;
                case mot.rueck: daten[1] = 0x05; break;
                case mot.links: daten[1] = 0x02; break;
                case mot.rechts: daten[1] = 0x08; break;
                case mot.linksdreh: daten[1] = 0x06; break;
                case mot.rechtsdreh: daten[1] = 0x09; break;
                case mot.linksrueck: daten[1] = 0x04; break;
                case mot.rechtsrueck: daten[1] = 0x01; break;
                default: daten[1] = 0x00; break;
            }

            if (speed > 10 || speed < 0)
                error("ERROR: 1", 8);                           //Sicherstellen, dass Speed
            else                                                //gültigen Parameter enthält
            {
                temp = Convert.ToInt32(speed * 25.5);
                daten[2] = Convert.ToByte(temp);     //"+" Vorzeichen: Geschwindigkeit aufbereiten
                SendeDaten(daten);
            }
        }

        private void SendeDaten(byte[] daten)
        {
            serialPort1.Write(daten, 0, 1);                 //Senden der Daten über RS-232 an Roboter
            //Thread.Sleep(Warte);                            //Pause von "Warte" ms um sicherzustellen
            serialPort1.Write(daten, 1, 1);                 //dass der Roboter alle Daten korrekt erhält
            //Thread.Sleep(Warte);
            serialPort1.Write(daten, 2, 1);
            Thread.Sleep(Warte);
        }

        //Es wird der Wert des angesprochenen Analog Port zurückgegeben
        //PortNr        --> 0 - 3 entspicht Ain0 ... Ain3 des PCF8551
        //Rückgabewert  --> 0...255 = 0V...5V
        public int getAnalogPort(byte PortNr)
        {
            byte[] daten = new byte[3];
            int rueckgabe = 0;

            if (PortNr > 3 || PortNr < 0)                       //Sicherstellen, dass PortNr
            {                                                   //erlaubte Parameter enthält
                error("ERROR: 2", 8);
                return -1;
            }
            else
            {

                daten[0] = 4;                                   //4 spricht im Roboter die Analogportabfrage an
                daten[1] = PortNr;                              //Angesprochener Port im Roboter
                daten[2] = 1;                                   //Notwendig, da Roboter  3 Bytes erwartet

                SendeDaten(daten);

                rueckgabe = serialPort1.ReadByte();             //Empfängt Wert des AnalogPort von Roboter

                return rueckgabe;
            }
        }

        //Es wird der Wert als Spannung am Analog-Ausgangs ausgegeben
        //AnalogWert  --> 0...255 = 0V...5V
        public void setAnalogPort(byte AnalogWert)
        {
            byte[] daten = new byte[3];

            if (AnalogWert > 255 || AnalogWert < 0)             //Sicherstellen, dass AnalogWert
            {                                                   //erlaubte Parameter enthält
                error("ERROR: 3", 8);
            }
            else
            {

                daten[0] = 10;                                  //10 spricht im Roboter die Analog-ausgabe an
                daten[1] = AnalogWert;                          //Analoger Ausgabewert am Roboter
                daten[2] = 1;                                   //Notwendig, da Roboter  3 Bytes erwartet

                SendeDaten(daten);
            }
        }

        //Es werden die Sensoren eingelesen und 
        //als Variable vom Typ enum Sens zurückgegeben!     
        public Sens readSensors()
        {
            byte[] daten = new byte[3];

            daten[0] = 11;                                  //11 spricht im Roboter die Senorabfrage an
            daten[1] = 1;                                   //Notwendig, da Roboter  3 Bytes erwartet
            daten[2] = 1;                                   //    "

            SendeDaten(daten);                              // Sensorlese-Befehl senden

            byte buffer = (byte)serialPort1.ReadByte();   // Sensoren Rücklesen

            return (Sens)buffer;                           // Byte in Enum-Variable vom Typ Sens konvertieren
        }

        //Es wird der Wert des angesprochenen Digital Port zurückgegeben
        //PortNr        --> 0 - 15      (P1.0 ... P1.7, P2.0 ... P2.7)
        //Rückgabewert  --> 0 - 1
        public int getPort(byte PortNr)
        {
            byte[] daten = new byte[3];
            int rueckgabe = 0;

            if (PortNr > 15 || PortNr < 0)                      //Sicherstellen, dass PortNr
            {                                                   //erlaubte Parameter enthält
                error("ERROR: 3", 8);
                return -1;
            }
            else
            {

                daten[0] = 2;                                   //2 spricht im Roboter die Digitalportabfrage an
                daten[1] = PortNr;                              //Angesprochener Port im Roboter
                daten[2] = 1;                                   //Notwendig, da Roboter 3 Bytes erwartet

                SendeDaten(daten);

                rueckgabe = serialPort1.ReadByte();             //Empfängt Wert des DigitalPort von Roboter

                return rueckgabe;
            }
        }

        //Angesprochener Digital Port kann gesetzt werden
        //PortNr        --> 1 - 21
        //Wert          --> 0 - 1
        public void setPort(byte PortNr, byte Wert)
        {
            byte[] daten = new byte[3];

            if (PortNr > 15 || PortNr < 0 || Wert > 1 || Wert < 0)//Sicherstellen, dass PortNr und Wert
                error("ERROR: 4", 8);                           //erlaubte Parameter enthalten
            else
            {
                daten[0] = 3;                                   //3 spricht im Roboter das setzen eines  Port an
                daten[1] = PortNr;                              //Angesprochener Port im Roboter
                daten[2] = Wert;                                //Zu setzender Wert

                SendeDaten(daten);
            }
        }

        public void Beep(byte On)
        {
            byte[] daten = new byte[3];

            if (On > 1 || On < 0)              //Sicherstellen, dass On 1 oder 0 ist
                error("ERROR: 12", 9);
            else
            {
                daten[0] = 12;                  //12 spricht im Roboter die Beep-Routine an.
                daten[1] = On;                  //Angesprochener Port im Roboter
                daten[2] = 0;                   //Dummy

                SendeDaten(daten);
            }
        }

        //Es werden die Motoren angehalten, ein Fehlercode
        //und ein akustisches Signal am Roboter ausgegeben
        private void error(String Text, int Laenge)
        {
            motor(mot.stopp, 0);                                //Motoren stoppen
            for (int i = 0; i < 5; i++)
            {
                Beep(1);
                Thread.Sleep(300);
                Beep(0);
                Thread.Sleep(300);
            }
            System.Windows.Forms.MessageBox.Show(Text);         //Fehlercode ausgeben!
        }
    }
}


//********************  Fehlerliste  ******************
//
//  ERROR 1: Ungültiger Parameter "speed" in der Methode "motor"
//  ERROR 2: Ein ungültiger Parameter in der Methode "getAnalogPort"
//  ERROR 3: Ein ungültiger Parameter in der Methode "getPort"
//  ERROR 4: Ein oder mehrere ungültige Parameter in der Methode "setPort"
//  ERROR 12: Ungültiger Parameter "On" in der Methode "Beep"
